package io.ars;

import salsa.language.Actor;
import salsa.language.ActorReference;
import salsa.language.Message;

import java.util.Hashtable;
import java.util.Vector;

import io.core.agents.ProfilingAgent;

public class ARSProfilingAgent implements ProfilingAgent {

	private Hashtable actorProfiles = new Hashtable();
	private long totalProcessed = 0;
	private long processedSinceStart = 0;

	// supposed to be cycles/s (The processing value)
	public double getProcessing() 			{ return 100; }
	public long processed()				{ return totalProcessed; }

	public Vector getActorProfiles() 		{ return new Vector(actorProfiles.values()); }
	public Vector getActorReferences() {
		Vector profiles = getActorProfiles();
		Vector references = new Vector();

		for (int i = 0; i < profiles.size(); i++) {
			references.add( new ActorReference(((Actor) ((ActorProfile)profiles.get(i)).getActor()).getUAN()) );
		}
		return references;
	}

	int maxMailboxSize = 0;
	int currentMailboxSize = 0;
	public boolean isLight() 			{ return (currentMailboxSize <= maxMailboxSize); }

	boolean waiting = false;
	public boolean isWaiting() 			{ return waiting; }
	public void setWaiting(boolean waiting) 	{ this.waiting = waiting; }

	boolean computing = true;
	public boolean isComputing()			{ return computing; }
	public void setComputing(boolean computing)	{ this.computing = computing; }

	public ARSProfilingAgent() {}

	public void beginProcessed(String identifier) {
		totalProcessed++;
		processedSinceStart++;

		ActorProfile target = (ActorProfile)actorProfiles.get(identifier);
		if (target == null) return;  
    
		target.processedMessage();
	}
	public void endProcessed(String identifier) { currentMailboxSize--; }

	public void received(String identifier, Message message) {
		ActorProfile target = (ActorProfile)actorProfiles.get(identifier);
		if (target == null) return;

		if (message.getTarget().ual != null) {
			target.incrementReceived( message.getTarget().ual.getLocation() );
		}

		currentMailboxSize++;
		if (currentMailboxSize > maxMailboxSize) maxMailboxSize = currentMailboxSize;
	}

	public void beginSend(String identifier, Message message) {
		ActorProfile target = (ActorProfile)actorProfiles.get(identifier);
		if (target == null) return;    		
		target.incrementSent( message.getTarget().ual.getLocation() );
	}
	public void endSend(String identifier, Message message) { }

	/*
		The following create new actor profiles and manage their
		entering and exiting the theater
	*/
	public void actorEnter(String identifier) 	{ actorProfiles.put(identifier, new ActorProfile(identifier)); }
	public void actorExit(String identifier) 	{ actorProfiles.remove(identifier); }
	public void updateIdentifier(String uanId, String ualId) {
		ActorProfile target = (ActorProfile)actorProfiles.get(ualId);
		actorProfiles.remove(ualId);
		actorProfiles.put(uanId, target);
	}

	public void reset() {
		totalProcessed = 0;
		maxMailboxSize = 0;
		currentMailboxSize = 0;

		Vector profiles = getActorProfiles();
		for (int i = 0; i < profiles.size(); i++) { ((ActorProfile)profiles.get(i)).reset(); }
	}

	public long totalProcessed() { return processedSinceStart; }
}
