package io.ars;

import io.core.language.AutonomousActor;

import java.util.Hashtable;

import salsa.language.Message;
import salsa.language.ServiceFactory;
import salsa.messaging.TheaterService;
import salsa.naming.UAL;
import salsa.naming.UAN;

public class ActorProfile {
	private UAN uan;
	private UAL ual;

	private boolean scheduled = false;
	private long processed = 0;
	private long lastResetTime = System.currentTimeMillis();
	private Hashtable targetList = new Hashtable();
	private Hashtable sourceList = new Hashtable();

	private TheaterService theaterService = ServiceFactory.getTheater();

	public long processed()			{ return processed; }
	public long lastResetTime()		{ return lastResetTime; }
	public boolean getScheduled()		{ return scheduled; }
	public void setScheduled(boolean s)	{ scheduled = s; }
	
	public UAN getUAN() { 
		if (uan == null) {
			if (theaterService.getEntry(ual) instanceof UAN) {
				uan = (UAN)theaterService.getEntry(ual);
			}
		}
		return uan; 
	}

	public ActorProfile(String identifier) {
		if (identifier.substring(0,4).equals("rmsp")) 	ual = new UAL(identifier);
		else 						uan = new UAN(identifier);
	}

	public void reset() {
		processed = 0;
		lastResetTime = System.currentTimeMillis();

		sourceList = new Hashtable();
		targetList = new Hashtable();
	}

        /*
		This retrives the actor from whichever registry it resides in
		if the actor is not anonymous (it has a UAN), it will get the
		state from the non-anonymous registry.  If the actor is foreign,
		this registry returns the apropriate ual
		If the actor is anonymous, it will get the state from the anonymous
		registry. If that actor has since become non-anonymous (through the bind method)
		that registry will return the UAN, and this reference will become non-anonymous
	*/
	public AutonomousActor getActor() {
		Object state = null;

		if (uan != null) {
			state = theaterService.getEntry(uan);
		} else if (ual != null) {
			state = theaterService.getEntry(ual);
			if (state instanceof UAN) {
				uan = (UAN) state;
				state = theaterService.getEntry(uan);
			}
		}

		if (state instanceof AutonomousActor) return (AutonomousActor)state; 
		else return null;
	}

	public void processedMessage() { processed++; }

	public int receivedFrom(String targetLocation) {
		int receivedFrom = 0;

		if (sourceList == null) sourceList = new Hashtable();
		Object rec = sourceList.get(targetLocation);
		if (rec != null) receivedFrom = ((Integer)rec).intValue();

		return receivedFrom;
	}

	public int sentTo(String targetLocation) {
		int sentTo = 0;

		if (targetList == null) targetList = new Hashtable();
		Object sent = targetList.get(targetLocation);
		if (sent != null) sentTo = ((Integer)sent).intValue();

		return sentTo;
	}

	public void incrementReceived(String targetLocation) {
		if (sourceList == null) sourceList = new Hashtable();

		/* add the target location to our list of peers and number of messages received from them */
		Object receivedFrom = sourceList.get(targetLocation);
		if (receivedFrom != null) receivedFrom = new Integer( ((Integer)receivedFrom).intValue() + 1 );
		else receivedFrom = new Integer(1);

		sourceList.put(targetLocation, receivedFrom);
	}


	public void incrementSent(String targetLocation) {
		if (targetList == null) targetList = new Hashtable();

		/* add the target location to our list of peers and number of messages sent to them */
		Object sentTo = targetList.get(targetLocation);
		if (sentTo != null) sentTo = new Integer( ((Integer)sentTo).intValue() + 1 );
		else sentTo = new Integer(1);

		targetList.put(targetLocation, sentTo);
	}       
}
