/*
 *	Modified Extensively for SALSA 0.6.0 new token semantics.
 *
 *	Author: Travis Desell
 *
 *	Generates the input and output tokens as generated by the salsa
 *	compiler's message sending code.
 */

package salsa.language;

import java.util.Vector;
import salsa.messaging.TheaterService;

public class Token {

	private String tokenName;
	private Vector inputTokens = new Vector();
	private boolean hasValue = false;
	private boolean isJoinToken = false;
	private Object value = null;

	private TheaterService theater = ServiceFactory.getTheater();

	public Token() {
		tokenName = "unnamed";
	}

	public Token(String id) {
		tokenName = id;
	}

	public Token(String id, Object value) {
		tokenName = id;
		hasValue = true;
		this.value = value;
	}

	public void setJoinToken() {
		isJoinToken = true;
	}

	public TokenInput createInput() {
		TokenInput newInput = new TokenInput();
		inputTokens.add( newInput );
		return newInput;
	}

	public Object createOutput(ActorReference target, Message containingMessage, int position) {
		if (hasValue) {
			return value;
		} else {
			if (inputTokens.size() == 0) {
				System.err.println("Error: trying to create a token without a value");
				System.err.println("\ttoken used in message: " + containingMessage.getName());
			}

			String outputId = theater.getUniqueTokenId() + "_" + tokenName + "_" + containingMessage.getMethodName();

			for (int i = 0; i < inputTokens.size(); i++) {
				((TokenInput)inputTokens.get(i)).addTarget( target, outputId );
			}

			if (inputTokens.size() == 1 && !isJoinToken) {
				return new TokenOutput(outputId, containingMessage, position);
			} else {
				return new TokenOutput(outputId, containingMessage, position, inputTokens.size());
			}
		}
	}
}
