/*
 *	Created for SALSA 0.6.0 new token semantics
 *	(July, 2003)
 *
 *	Author: Travis Desell
 *
 *	TokenOutput is created by the generated salsa code to act as a placeholder
 *	inside a messages arguments, until it's real value has been resolved and
 *	sent to the actor who will process that message (in a TokenInput).
 */

package salsa.language;

public class TokenOutput implements java.io.Serializable {

	/*
	 *	A reference to the message which contains this TokenOutput as a
	 *	placeholder in its arguments for an incoming token value
	 */
	public Message containingMessage = null;

	/*
	 *	The position of this TokenOutput in its containing arguments.
	 *	-1 specifies that the TokenOutput is not contained by any arguments,
	 *	but rather used only for synchronization of messages in a
	 *	continuation.
	 */
	private int position = -1;

	/*
	 *	The number of input values that must be received before this
	 *	token can be completely resolved.  This is used in waiting for
	 *	all return values of messages sent within a join block.
	 */
	protected int requiredValues = -1;
	private Object[] values = null;


	/*
	 *	An id unique to this TokenOutput, it's creating Token, and all of its
	 *	TokenInputs, used for resolving a value sent by a token input
	 */
	private String uniqueId;
	public String getId()	{ return uniqueId; }

	/*
	 *	The constructor for TokenOutput, as used by Token in the method
	 *	createOutput.
	 */
	public TokenOutput(String uniqueId, Message containingMessage, int position) {
		this.containingMessage = containingMessage;
		this.position = position;
		this.uniqueId = uniqueId;
	}

	/*
	 *	Another constructor for TokenOutput, for when there are multiple inputs
	 *	to this TokenOutput.
	 */
	public TokenOutput(String uniqueId, Message containingMessage, int position, int requiredValues) {
		this.containingMessage = containingMessage;
		this.position = position;
		this.uniqueId = uniqueId;
		this.requiredValues = requiredValues;
		values = new Object[requiredValues];
	}


	/*
	 *	Replaces this token in the arguments with its appropriate value
	 */
	public void replace(Object value) {
		if (System.getProperty( "debug") != null)
			System.err.println("REPLACE: " + this.uniqueId + " for " + containingMessage.getName());

		if (requiredValues == -1) {
			if (position == -1) {
				containingMessage.setInput(value);
			} else {
				containingMessage.arguments[position] = value;
			}
			containingMessage.receivedToken();
		} else {
			requiredValues--;
			values[requiredValues] = value;

			if (requiredValues == 0) {
				if (position == -1) {
					containingMessage.setInput(value);
				} else {
					containingMessage.arguments[position] = values;
				}
				containingMessage.receivedToken();
			}
		}
	}
}
