package salsac;

/**
 * $Id: SalsaCompiler.java,v 1.18 2003/09/09 01:00:44 deselt Exp $
 *
 * This is the Salsa programming language compiler.  Compilation goes
 * through two major steps:
 *
 * (1) Parsing phase:  Here we call the Salsa Parser.
 *                     As generated by JavaCC with a few changes to keep
 *                     the input for code generation.
 *
 * (2) Code Generation phase:  Here we use the abstract syntax tree
 *                             produced by the first phase to build 
 *                             Java files corresponding to the original
 *                             Salsa files being compiled.
 * Modified by Robin D. Toll	2001/12
 *		-Re-coded parameter handling.
 * Modified by Travis Desell	2003/7
 *		-Re-write of the grammer and code generation for salsa 0.6.0
 *
 * @author Carlos Varela   cvarela@uiuc.edu   1999/09/14
 * @version 0.3.3 2002/04/02
 */

import java.io.*;

public class SalsaCompiler {
	public static final String VERSION = "0.6.07";
	private String[] files = null;

        public static SymbolTable symbolTable = new SymbolTable();
        public static int indent = 0;

        public static String getIndent() {
                String code = "";
                for (int i = 0; i < indent; i++) {
                        code += "\t";
                }
                return code;
        }


	public static void main(String args[]) {
		new SalsaCompiler(args);
	}

	SalsaCompiler(String[] args) {
		SalsaParser parser = null;
		String outputFile = null;

		if (args.length == 0) {
			System.out.println("Salsa Compiler Version " + VERSION + ":  Reading from standard input . . .");
			parser = new SalsaParser(System.in);
		} else {
			for (int x = 0; x < args.length; x++) {
				if (args[x].equals("-?")) help();
				else addFile( args[x] );
			}

			if ((files == null) || (files.length < 1)) {
				System.out.println("Salsa Compiler Version " + VERSION + ":  Reading from standard input . . .");
				parser = new SalsaParser(System.in);
			}

			for (int x = 0; x < files.length; x++) {
				System.out.println();
				System.out.println("Salsa Compiler Version " + VERSION + ":  Reading from file " + files[x] + " . . .");

				try {
					if (x == 0) {
						parser = new SalsaParser(new java.io.FileInputStream(files[x]));
					} else {
						parser.ReInit(new java.io.FileInputStream(files[x]));
					}

					if ( !files[x].endsWith(".salsa") ) {
						System.out.println("Salsa Compiler Version " + VERSION + ":  File must end with \".salsa\"");
					} else { // Okay, it's passed all our tests; lets actually compile it.
						outputFile = files[x].substring(0, files[x].length()-6).concat(".java");
					}

					try {
						SimpleNode n = parser.CompilationUnit();
						System.out.println("Salsa Compiler Version " + VERSION + ":  Salsa program parsed successfully.");

						if (outputFile != null) {
							// cvarela: if it exists, it should rename it...
							new FileOutputStream(outputFile).write( n.getJavaCode().getBytes() );
						} else {
							System.out.println( n.getJavaCode() );
						}

						System.out.println("Salsa Compiler Version " + VERSION + ":  Salsa program compiled successfully.");
					} catch (IOException e) {
						System.out.println(e.getMessage());
						System.out.println("Salsa Compiler Version " + VERSION + ":  Encountered errors writing to file:"+outputFile);
					} catch (ParseException e) {
						System.out.println(e.getMessage());
						System.out.println("Salsa Compiler Version " + VERSION + ":  Encountered errors during parse.");
					}
				} catch (java.io.FileNotFoundException e) {
					System.out.println("Salsa Compiler Version " + VERSION + ":  File " + files[x] + " not found.");
				}

			}
		}
	}

	private static void help() {
		System.out.println("Salsa Compiler Version " + VERSION + ":  Usage is one of:");
		System.out.println("         java SalsaCompiler < inputfile");
		System.out.println("OR");
		System.out.println("         java SalsaCompiler inputfile");
		System.exit(1);
	}

	private void addFile(String fn) {
		if (files == null) {
			files = new String[1];
			files[0] = fn;
		} else {
			String[] tmp = new String[files.length+1];
			System.arraycopy(files, 0, tmp, 0, files.length);
			tmp[files.length] = fn;
			files = tmp;
		}
	}
}
