/*
	Code generation for the following grammar segment:
	
	ConstructorDeclaration :=
		MethodAttributes() <IDENTIFIER> FormalParameters() ["throws" Exceptions()] "{" [ExplicitConstructorInvocation()] Statement()* "}"
*/

package salsac.definitions;

import salsac.*;
import salsac.SalsaCompiler;

public class ConstructorDeclaration extends SimpleNode {
	public ConstructorDeclaration(int id) 			{ super(id); }
	public ConstructorDeclaration(SalsaParser p, int id)	{ super(p, id); }

	private int i = 0;

	public String getPreCode() 	{ 
		//a new block creates a lower level of the symbol table
		SalsaCompiler.symbolTable = new SymbolTable(SalsaCompiler.symbolTable);
		SalsaCompiler.indent++;

		String code = getChild(0).getJavaCode();
		if (!code.equals("")) code += " ";

		code += "State" + getChild(1).getJavaCode();
		
		i = 2;
		if (getToken(1).image.equals("throws")) {
			code += " throws " + getChild(2).getJavaCode();
			i = 3;
		}

		return code + " {\n";
	}

	public String getPostCode() {
		//move back up the symbol table as we leave the block
		SalsaCompiler.symbolTable = SalsaCompiler.symbolTable.parent;
		SalsaCompiler.indent--;

		return SalsaCompiler.getIndent() + "\tServiceFactory.getTransport().send(messages);\n" +
		       SalsaCompiler.getIndent() + "}\n";
	}

	public String getChildCode() {
		String code = "";

		if (getChild(i) instanceof ExplicitConstructorInvocation) {
			code += SalsaCompiler.getIndent() + getChild(i).getJavaCode();
			i++;
		}

		code += SalsaCompiler.getIndent() + "self.ual = __ual;\n";
		code += SalsaCompiler.getIndent() + "Vector messages = new Vector();\n\n";

		for (; i < children.length; i++) {
			code += SalsaCompiler.getIndent() + getChild(i).getJavaCode();
		}

		return code;
	}
	
	
}
