package wwc.messaging;

import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.Vector;

import salsa.language.*;
import salsa.language.exceptions.*;
import salsa.naming.*;
import salsa.messaging.*;

/**
 * This class is the reference implementation of the MessagingService. The
 * service is implemented as a RMSP client.
 * 
 * This implementation uses the NamingService interface and the salsa.language.
 * ServiceFactory for UAN resolution.
 * 
 * @author stepha
 */
public class Messenger implements TransportService {
	/**
	 * Constructor for Messenger.
	 */
	public Messenger() {
		super();
	}

	public void migrate(Actor state, UAL target) {
		Socket outgoing = null;
		try {
			outgoing = new Socket(target.getHost(), target.getPort());
			ObjectOutputStream outStream = new ObjectOutputStream(outgoing.getOutputStream());
			outStream.writeObject( state );
			outStream.flush();
		} catch (IOException e) {
			throw new SalsaWrapperException("Error migrating actor: ", e);
		}

		// Wait for the Acknowledgement that the actor is situated on the remote theater.
		try {
			RMSPAcknowledgement ack = (RMSPAcknowledgement) new ObjectInputStream( outgoing.getInputStream() ).readObject();
		} catch (Exception e) {
			throw new SalsaWrapperException( "Did not recieve RmspAcknowledgement. ", e);
		}

		try {
			outgoing.close();
		} catch (IOException e) {
			throw new SalsaWrapperException("Error closing socket: ", e);
		}
	}

	public void send(Message message, UAL target) {
		// Determine the destination of this message.
		if (target == null) {
			UAN uan = message.getTarget().getUAN();

			if (uan != null) {
				Object entry = ServiceFactory.getTheater().getEntry(uan);
				if ((entry instanceof UAL) || (entry == null)) {
					// Resolve the location.
					target = ServiceFactory.getNaming().resolve( uan );
					ServiceFactory.getTheater().setEntry(uan, target);
				} else {
					((Actor)entry).send(message);
					return;
				}
			} else {
				System.err.println("UAN and UAL of message are null, cannot send message");
			}
		}

		try {
			Socket outgoing = new Socket(target.getHost(), target.getPort());
			ObjectOutputStream outStream = new ObjectOutputStream(outgoing.getOutputStream());
			outStream.writeObject( message );
			outStream.flush();
			outStream.close();
			outgoing.close();
		} catch (IOException e) {
			throw new SalsaWrapperException("Error sending message: ", e);
		}
	}

	public void send(Vector messages) {
		//used by the compiler to send a vector of messages generated by a message

		for (int i = 0; i < messages.size(); i++) {
			final Message current = (Message)messages.get(i);

			final ActorReference target = current.getTarget();
			try {
				if (current.getProperty() != null && current.getProperty().equals("delay")) {
					//This supports sending messages with the delay property
					Object[] parameters = current.getPropertyParameters();
					final long sleepTime = ((Integer)parameters[0]).intValue();

					Runnable delayThread = new Runnable() {
						public void run() {
							try {
								Thread.sleep(sleepTime);
							} catch (Exception e) {
								System.err.println("Error in delay thread for message: " + current.getName());
							}
							target.send(current);
						}
					};
					new Thread(delayThread, "Delay Thread").start();
				} else {
					target.send( current );
				}
			} catch (Exception e) {
				System.err.println("Error sending message: " + current.getName());
				System.err.println("\t" + e);
			}
		}
	}
}
