/** SALSA/World Wide Computer Project
 *  ObjectHandler recieves incoming objects
 *  and determines if they are actors or messages.
 *  upon recieving actors, it will start them up.
 *  messages get sent to the appropriate actor.
 *
 *  author Travis Desell
 */
package wwc.messaging;

import java.net.Socket;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;

import salsa.language.Actor;
import salsa.language.Message;
import salsa.language.RunTime;
import salsa.language.ServiceFactory;
import salsa.messaging.*;

public class PersistentHandler implements ReceptionService {
	TheaterService theater;

	public PersistentHandler() {
		theater = ServiceFactory.getTheater();
	}

	public void process(Socket incoming) {
		// Read in the object.
		Object received = null;
		ObjectInputStream inStream = null;
		ObjectOutputStream outStream = null;

		while (true) {
			if (inStream == null) {
				try {
					inStream = new ObjectInputStream( incoming.getInputStream() );
				} catch (IOException e) {
					System.err.println( "IOException occured opening input stream: " + e.getMessage() );
					break;
				}
			}

			try {
				received = inStream.readObject();
			} catch (ClassNotFoundException e) {
				System.err.println( "Could not load class for " + received.getClass().getName() );
				System.err.println( "\tIs it in the theater CLASSPATH?" );
				System.err.println( e.getMessage() );
				break;
			} catch (IOException e) {
				System.err.println( "IOException occured reading object " + e.getMessage() );
				break;
			}

			if (received instanceof Actor) {
				Actor incomingActor = (Actor)received;

		                //add the state to the stateRegistry, so incoming actor references can
		                //get to it
		                theater.setEntry(incomingActor.getUAN(), incomingActor);
		                RunTime.receivedUniversalActor();

		                // Send an RmspAcknowledgement.
				if (outStream == null) {
			                try {
						outStream = new ObjectOutputStream( incoming.getOutputStream() );
					} catch (IOException e) {
						System.err.println( "Could not send open an ObjectOutputStream to remote theater: " + e.getMessage() );
					}
				}
				try {
					outStream.writeObject( new RMSPAcknowledgement() );
				} catch (IOException e) {
					System.err.println( "Could not send RmspAcknowledgement is the remote theater compliant? " + e.getMessage() );
		                }
			} else if (received instanceof Message) {
				Message message = (Message) received;
				message.getTarget().send(message);
			}
		}

		// Close the connection.
		try {
			incoming.close();
		} catch (IOException e) {
			System.err.println( "IOException occured closing connection" + e.getMessage() );
			return;
		}
	}
}
