package wwc.messaging; 

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Hashtable;
import java.util.PropertyResourceBundle;
import java.util.Vector;

import salsa.language.Actor;
import salsa.language.ServiceFactory;
import salsa.language.exceptions.SalsaWrapperException;
import salsa.messaging.ReceptionService;
import salsa.messaging.TheaterService;
import salsa.naming.UAL;
import salsa.naming.UAN;


//very bad need to fix this for IOTheater to run silently.
import io.IOTheater;


/**
 * The Theater is the reference implementation of the TheaterService
 * interface.
 *
 * @version $Id: Theater.java,v 1.19 2003/09/09 22:25:15 deselt Exp $
 *
 * @author stepha
 */
public class Theater implements TheaterService {
	private static Hashtable registry = new Hashtable();
	private static Hashtable anonymousRegistry = new Hashtable();
	
	public static String version;
	
	private ServerSocket server;
	private static int port;

	private static int generatedTokens;
	private static int generatedLocations;
	private PropertyResourceBundle configFile;

	/**
	 * This method provides a way to generate a unique identifier
	 * for every token used by a SALSA program.
	 */
	public synchronized String getUniqueTokenId() {
		String generated = null;

		if (Theater.port == 0) {
			startTheater(0, null);
		}

		try {
			if (rmspString == null) {
				rmspString = "rmsp://" + InetAddress.getLocalHost().getHostName() + ":" + Theater.port;
			}
			generated = rmspString + "/" + (++Theater.generatedTokens);
		} catch (UnknownHostException e) {
			System.err.println("Could not generate localhost: " + e);
		}

		return generated;
	}

	/**
 	 * This method provides synchronization on the generated locations variable
 	 * to ensure that each requesting actor gets a unique location.
 	 * 
 	 * @return String
 	 */
	private String rmspString = null;
	public synchronized UAL generateUAL() {
		UAL generated = null;

		if (Theater.port == 0) {
			startTheater(0, null);
		}

		try {
			if (rmspString == null) {
				rmspString = "rmsp://" + InetAddress.getLocalHost().getHostName() + ":" + Theater.port;
			} 
			generated = new UAL( rmspString + "/location" + (++Theater.generatedLocations) );
		} catch (UnknownHostException e) {
			System.err.println("Could not generate localhost: " + e);
		}

		return generated;
	}

	/**
	 * Constructor for Theater.
	 */
	public Theater() {
		version  = "$Id: Theater.java,v 1.19 2003/09/09 22:25:15 deselt Exp $";
		server = null;
		Theater.port = 0;
		generatedLocations = 0;
		generatedTokens = 0;
	}

	public Object getEntry(UAL ual) {
		synchronized( Theater.anonymousRegistry ) {
			return Theater.anonymousRegistry.get(ual.toString());
		}
	}

	public void setEntry(UAL ual, Object entry) {
		synchronized( Theater.anonymousRegistry ) {
			Theater.anonymousRegistry.put(ual.toString(), entry);
		}
	}

	public void removeEntry(UAL ual) {
		synchronized( Theater.anonymousRegistry ) {
			Theater.anonymousRegistry.remove(ual.toString());
		}
	}

	public Object getEntry(UAN uan) {
		synchronized( Theater.registry ) {
			return Theater.registry.get(uan.toString());
		}
	}

	public void setEntry(UAN uan, Object entry) {
		synchronized( Theater.registry ) {
			Theater.registry.put(uan.toString(), entry);
		}
	}

	public void removeEntry(UAN uan) {
		synchronized( Theater.registry ) {
			Theater.registry.remove(uan.toString());
		}
	}


	public Hashtable getRegistry() {
		synchronized( Theater.registry ) {
			return Theater.registry;
		}
	}

	/**
	 * @see salsa.language.TheaterService#startTheater(int, salsa.language.ActorReference, java.lang.String)
	 */
	public TheaterService startTheater(int port, Actor initial) {
		if (server == null) {
			String pPort = System.getProperty("port");
			if (pPort != null) port = Integer.parseInt(pPort);
			Theater.port = port;

			// Create a server socket.
			try {
				server = new ServerSocket( Theater.port );
			} catch (IOException e) {
				throw new SalsaWrapperException( "Could not start theater.", e );
			}
			
			// Check to see what the port actually is.
			if (Theater.port == 0) {
				Theater.port = server.getLocalPort();
			}
			
			final TheaterService theater = this;
			final ServerSocket passive = server;
			
			// Accept connections.
			Runnable listenLoop = new Runnable() {
				public void run() {
					int i = 0;
					while (true) {
						try {
							final Socket incoming = passive.accept();

							Runnable handlerThread = new Runnable() {
								public void run() {
									ReceptionService handler = ServiceFactory.getReception();
									handler.process(incoming);
								}
							};
							new Thread(handlerThread, "Handler Loop " + i).start();
						} catch (IOException e) {
							throw new SalsaWrapperException( "Failed to accept connection.", e );
						}
						
						i++;
					}
				}
			};
	
			// Start the listening thread.
			new Thread( listenLoop, "Listen Loop Thread" ).start();
		}

		// Add the specified actor to the theater.
		if (initial != null) setEntry( initial.getUAN(), initial );

		return this;
	}


	/**
	 * @see salsa.language.TheaterService#getPort()
	 */
	public int getPort() {
		return Theater.port;
	}


	/**
	 * used by subclassed theaters to modify the port value (from command line arguments)
	 */
	public void setPort(int _port) {
		Theater.port = _port;
	}


	public static void main( String[] args ) {
		Thread.currentThread().setName( "Main startup thread.");
		
		// Check arguments.
		if (args.length > 1) {
			System.out.println( "Theater TheaterService Reference Implementation.");
			System.out.println( "Worldwide Computing Lab. Rensselaer Polytechic Institute." );
			System.out.println( "Version: " + version );
			System.out.println( "Usage: wwc.theater.Theater [<port>]" );
			System.out.println( "or invoke a Actor with VM option -Dtheater=wwc.theater.Theater" );
			return;
		}
		
		// Determine port.
		Theater.port = 0;

		String pPort = System.getProperty("port");
		if (pPort != null) Theater.port = Integer.parseInt(pPort);
		if (args.length == 1) {
			try {
				Theater.port = Integer.parseInt( args[0] );
			} catch (NumberFormatException e) {
				System.out.println( "Bad port." );
				System.out.println( "Theater TheaterService Reference Implementation.");
				System.out.println( "Worldwide Computing Lab. Rensselaer Polytechic Institute." );
				System.out.println( "Version: " + version );
				System.out.println( "Usage: wwc.theater.Theater [<port>]" );
				System.out.println( "or invoke a Actor with VM option -Dtheater=wwc.theater.Theater" );
				return;
			}
		}
		
		// Create a theater.
		Theater theater = new Theater();
		
		// Construct service factory.
		ServiceFactory.createFactory( theater, null, null, null );
		
		// Start the theater.
		theater.startTheater(Theater.port, null);

		try {
			InetAddress IPHere = InetAddress.getLocalHost();
			String stringHere = "rmsp://" + IPHere.getHostAddress() + ":" + Theater.port;
			if ( System.getProperty("silent") == null ) System.err.println("Theater started at: " + stringHere);
		} catch (UnknownHostException e) {
			System.err.println("Unkown local host? " + e);
		}
	}
}
